<?php
/**
 * Plugin Name: Wordpress Portfolio Plugin
 * Plugin URI: http://wordpress.org/extend/plugins/wp-portfolio/
 * Description: A plugin that allows you to show off your portfolio through a single page on your wordpress blog with automatically generated thumbnails. To show your portfolio, create a new page and paste [wp-portfolio] into it. The plugin requires you to have a free account with <a href="http://www.shrinktheweb.com/">Shrink The Web</a> to generate the thumbnails.
 * Version: 1.10
 * Author: Dan Harrison
 * Author URI: http://www.danharrison.co.uk 
 */
/**
 * This plugin is licensed under the Apache 2 License
 * http://www.apache.org/licenses/LICENSE-2.0
 */

/* Libaries */
include_once('wplib/utils_formbuilder.inc.php');
include_once('wplib/utils_tablebuilder.inc.php');
include_once('wplib/utils_sql.inc.php');



/** Constant: The string used to determine when to render the portfolio. */
define('WPP_STR_SHOW_PORTFOLIO', 				'/<!-- ShowMyPortfolio(\((([0-9],|[0-9]|,)*?)\))? -->/i');

/** Constant: The string used to determine when to render a group name. */
define('WPP_STR_GROUP_NAME', 					'%GROUP_NAME%');

/** Constant: The string used to determine when to render a group description. */
define('WPP_STR_GROUP_DESCRIPTION', 	 		'%GROUP_DESCRIPTION%');

/** Constant: The string used to determine when to render a website name. */
define('WPP_STR_WEBSITE_NAME', 	 				'%WEBSITE_NAME%');

/** Constant: The string used to determine when to render a website thumbnail image. */
define('WPP_STR_WEBSITE_THUMBNAIL', 	 		'%WEBSITE_THUMBNAIL%');

/** Constant: The string used to determine when to render a website url. */
define('WPP_STR_WEBSITE_URL', 	 				'%WEBSITE_URL%');

/** Constant: The string used to determine when to render a website description. */
define('WPP_STR_WEBSITE_DESCRIPTION', 	 		'%WEBSITE_DESCRIPTION%');

/** Constant: Default HTML to render a group. */
define('WPP_DEFAULT_GROUP_TEMPLATE', 			
"<h2>%GROUP_NAME%</h2>
<p>%GROUP_DESCRIPTION%</p>");

/** Constant: Default HTML to render a website. */
define('WPP_DEFAULT_WEBSITE_TEMPLATE', 			
"<div class=\"portfolio-website\">
    <div class=\"website-thumbnail\"><a href=\"%WEBSITE_URL%\" target=\"_blank\">%WEBSITE_THUMBNAIL%</a></div>
    <div class=\"website-name\"><a href=\"%WEBSITE_URL%\" target=\"_blank\">%WEBSITE_NAME%</a></div>
    <div class=\"website-url\"><a href=\"%WEBSITE_URL%\" target=\"_blank\">%WEBSITE_URL%</a></div>
    <div class=\"website-description\">%WEBSITE_DESCRIPTION%</div>
    <div class=\"website-clear\"></div>
</div>");

/** Constant: Default CSS to style the portfolio. */
define('WPP_DEFAULT_CSS',
".portfolio-website {
	padding: 10px;
	margin-bottom: 10px;
}
.website-thumbnail {
	float: left;
	margin: 0 20px 20px 0;
}
.website-thumbnail img {
	border: 1px solid #555;
	margin: 0;
	padding: 0;
}
.website-name {
	font-size: 12pt;
	font-weight: bold;
	margin-bottom: 3px;
}
.website-name a,.website-url a {
	text-decoration: none;
}
.website-name a:hover,.website-url a:hover {
	text-decoration: underline;
}
.website-url {
	font-size: 9pt;
	font-weight: bold;
}
.website-url a {
	color: #777;
}
.website-description {
	margin-top: 15px;
}
.website-clear {
	clear: both;
}");


/** Constant: The name of the table to store the website information. */
define('TABLE_WEBSITES', 						'WPPortfolio_websites');

/** Constant: The name of the table to store the website information. */
define('TABLE_WEBSITE_GROUPS', 					'WPPortfolio_groups');

/** Contstant: The path to use to store the cached thumbnails. */
define('WPP_THUMBNAIL_PATH',					'wp-portfolio/cache');

/** Contstant: The path to use to store the cached thumbnails. */
define('WPP_THUMB_DEFAULTS',					'wp-portfolio/thumbnail_blank_');

/** Constant: URL location for settings page. */
define('WPP_SETTINGS', 							'admin.php?page=WPP_show_settings');

/** Constant: URL location for settings page. */
define('WPP_DOCUMENTATION', 					'admin.php?page=WPP_show_documentation');

/** Constant: URL location for website summary. */
define('WPP_WEBSITE_SUMMARY', 					'admin.php?page=wp-portfolio/wp-portfolio.php');

/** Constant: URL location for modifying a website entry. */
define('WPP_MODIFY_WEBSITE', 					'admin.php?page=WPP_modify_website');

/** Constant: URL location for showing the list of groups in the portfolio. */
define('WPP_GROUP_SUMMARY', 					'admin.php?page=WPP_website_groups');

/** Constant: URL location for modifying a group entry. */
define('WPP_MODIFY_GROUP', 						'admin.php?page=WPP_modify_group');

/** Constant: The current version of the database needed by this version of the plugin.  */
define('WPP_VERSION', 							'1.10');


/**
 * Function: WPPortfolio_menu()
 *
 * Creates the menu with all of the configuration settings.
 */
add_action('admin_menu', 'WPPortfolio_menu');
function WPPortfolio_menu()
{
	add_menu_page('WP Portfolio - Summary of Websites in your Portfolio', 'WP Portfolio', 8, __FILE__, 'WPPortfolio_show_websites');
	
	add_submenu_page(__FILE__, 'WP Portfolio - Modify Website', 'Add New Website', 8, 'WPP_modify_website', 'WPPortfolio_modify_website');
	add_submenu_page(__FILE__, 'WP Portfolio - Modify Group', 'Add New Group', 8, 'WPP_modify_group', 'WPPortfolio_modify_group');
	add_submenu_page(__FILE__, 'WP Portfolio - Groups', 'Website Groups', 8, 'WPP_website_groups', 'WPPortfolio_show_website_groups');		
	add_submenu_page(__FILE__, 'WP Portfolio - General Settings', 'Portfolio Settings', 8, 'WPP_show_settings', 'WPPortfolio_showSettingsPage');
	add_submenu_page(__FILE__, 'WP Portfolio - Layout Settings', 'Layout Settings', 8, 'WPP_show_layout_settings', 'WPPortfolio_showLayoutSettingsPage');
	add_submenu_page(__FILE__, 'WP Portfolio - Documentation', 'Documentation', 8, 'WPP_show_documentation', 'WPPortfolio_showDocumentationPage');
	
	
	//add_submenu_page(__FILE__, 'WP Portfolio - Testing', 'Testing', 8, 'WPP_show_testing_page', 'WPPortfolio_show_testing_page');
}


/**
 * Determine if we're on a page just related to WP Portfolio in the admin area.
 * @return Boolean True if we're on a WP Portfolio admin page, false otherwise.
 */
function WPPortfolio_areWeOnWPPPage()
{
	if (isset($_GET) && isset($_GET['page']))
	{ 
		$currentPage = $_GET['page'];
		
		// This handles any WPPortfolio page.
		if ($currentPage == 'wp-portfolio/wp-portfolio.php' || substr($currentPage, 0, 4) == 'WPP_') {
			return true;
		}	
	}
	 
	return false;
}


/**
 * Function used to test regular expression variations for selecting only certain groups.
 */
/*
function WPPortfolio_show_testing_page()
{
	$idx = 0;
	$tests[$idx++] = '<!-- ShowMyPortfolio -->';
	$tests[$idx++] = '<!-- ShowMyPortfolio() -->';
	$tests[$idx++] = '<!-- ShowMyPortfolio(4) -->';
	$tests[$idx++] = '<!-- ShowMyPortfolio(4,) -->';
 	$tests[$idx++] = '<!-- ShowMyPortfolio(,) -->';
 	$tests[$idx++] = '<!-- ShowMyPortfolio(34,3,6) -->';
 	$tests[$idx++] = '<!-- ShowMyPortfolio(34,3,6,) -->';
 	$tests[$idx++] = '<!-- ShowMyPortfolio(1,23,45,67,8910) -->';
	
	?>
	<div style="margin: 20px; font-size: 8pt; line-height: 8pt;">
	<pre>
	<?php 
	
	foreach ($tests as $testidx => $test)
	{
		$testid = $testidx+ 1;
		$testcode = htmlentities($test);
		
		echo "\nTest $testid: $testcode)\n";
		
		preg_match(WPP_STR_SHOW_PORTFOLIO, $test, $matches);
		print_r($matches);
		echo "Found Values:\n";		
		print_r(explode(",", $matches[2]));
		echo "\n\n"; 
	}
			
	?>
	</pre></div>
	<?php	
} */

/**
 * Show the main settings page.
 */
function WPPortfolio_showSettingsPage() {

	// Hide tag upgrade message
	if (isset($_GET['hidetagupgrade']))
	{
		if ($_GET['hidetagupgrade'] == 'yes') { 
			update_option('WPPortfolio_setting_hide_tag_upgrade', 'true');
		} else {
			// Kept this here to allow it to be reversed if needed
			update_option('WPPortfolio_setting_hide_tag_upgrade', 'false');
		}
		
		// Reload the page, as notices is processed before the page loads, and it would appear to
		// the user that the hide did not work.
		?>
			<SCRIPT language="JavaScript">
    		<!--
                     top.location="<?php echo WPP_SETTINGS; ?>";
			//-->
			</SCRIPT>
		<?php 
	}	
	
?>
	<div class="wrap">
	<div id="icon-options-general" class="icon32">
	<br/>
	</div>
	<h2>WP Portfolio - General Settings</h2>
<?php 	

	// Get all the options from the database for the form
	$setting_stw_access_key = stripslashes(get_option('WPPortfolio_setting_stw_access_key'));
	$setting_stw_secret_key = stripslashes(get_option('WPPortfolio_setting_stw_secret_key'));
	$setting_stw_thumb_size = stripslashes(get_option('WPPortfolio_setting_stw_thumb_size'));
	$setting_cache_days 	= stripslashes(get_option('WPPortfolio_setting_cache_days'));
	$setting_fetch_method	= get_option('WPPortfolio_setting_fetch_method');
	$setting_show_credit	= get_option('WPPortfolio_setting_show_credit');
		
	// If we don't have the version in the settings, we're not installed
	if (!get_option('WPPortfolio_version')) {
		WPPortfolio_showMessage("No WP Portfolio settings were found, so it appears that the plugin has been uninstalled. Please <b>deactivate</b> and then <b>activate</b> the WP Portfolio plugin again to fix this.", true);
		return false;
	}
	
		
	// Uninstall plugin?
	if ($_GET['uninstall'] == "yes")
	{
		if ($_GET['confirm'] == "yes") {
			WPPortfolio_uninstall();
		}
		else {
			WPPortfolio_showMessage('Are you sure you want to delete all WP Portfolio settings and data? This action cannot be undone! </strong><br/><br/><a href="'.WPP_SETTINGS.'&uninstall=yes&confirm=yes">Yes, delete.</a> &nbsp; <a href="'.WPP_SETTINGS.'">NO!</a>');
		}
		return false;
	} // end if ($_GET['uninstall'] == "yes")	
	
	// Check if clearing cache 
	else if ( isset($_POST) && isset($_POST['clear_thumb_cache']) ) 
	{
		$actualThumbPath = WPPortfolio_getThumbPathActualDir();
		
		// Delete all contents of directory but not the root
		WPPortfolio_unlinkRecursive($actualThumbPath, false);
				
		WPPortfolio_showMessage("Thumbnail cache has now been emptied.");		
	}
	
	// Check if updated data.
	else if ( isset($_POST) && isset($_POST['update']) )
	{
		$setting_stw_access_key   = trim($_POST['setting_stw_access_key']);
		$setting_stw_secret_key   = trim($_POST['setting_stw_secret_key']);
		$setting_stw_thumb_size   = trim($_POST['setting_stw_thumb_size']);
		$setting_cache_days   	  = trim($_POST['setting_cache_days']);	
		$setting_fetch_method  	  = trim($_POST['setting_fetch_method']);
		$setting_show_credit	  = trim($_POST['setting_show_credit']);						
				
		// Validate keys
		if (WPPortfolio_isValidKey($setting_stw_access_key) && 
			WPPortfolio_isValidKey($setting_stw_secret_key))
		{		
			// Save settings
			update_option('WPPortfolio_setting_stw_access_key',   	$setting_stw_access_key);
			update_option('WPPortfolio_setting_stw_secret_key',   	$setting_stw_secret_key);
			update_option('WPPortfolio_setting_stw_thumb_size',   	$setting_stw_thumb_size);		
			update_option('WPPortfolio_setting_cache_days',   	  	$setting_cache_days);
			update_option('WPPortfolio_setting_fetch_method',     	$setting_fetch_method);
			update_option('WPPortfolio_setting_show_credit',     	$setting_show_credit);
								
			WPPortfolio_showMessage();		
		}
		else {
			WPPortfolio_showMessage("The keys must only contain letters and numbers. Please check that they are correct.", true);
		}
	}	
	
	
	$form = new FormBuilder();
	
	$formElem = new FormElement("setting_stw_access_key", "STW Access Key ID");
	$formElem->value = $setting_stw_access_key;
	$formElem->description = "The <a href=\"".WPP_DOCUMENTATION."#doc-stw\">Shrink The Web</a> Access Key ID is around 15 characters.";
	$form->addFormElement($formElem);
	
	$formElem = new FormElement("setting_stw_secret_key", "STW Secret Key");
	$formElem->value = $setting_stw_secret_key;
	$formElem->description = "The <a href=\"".WPP_DOCUMENTATION."#doc-stw\">Shrink The Web</a> Secret Key is around 5-10 characters. This key is never shared,<br>it is only stored in your settings and used to generate thumbnails for this website.";
	$form->addFormElement($formElem);
	
	// Thumbnail sizes
	$thumbsizes = array ("sm" => "Small (120 x 90)",
						 "lg" => "Large (200 x 150)",
						 "xlg" => "Extra Large (320 x 240)");
	
	$formElem = new FormElement("setting_stw_thumb_size", "Thumbnail Size");
	$formElem->value = $setting_stw_thumb_size;
	$formElem->setTypeAsComboBox($thumbsizes);
	$form->addFormElement($formElem);
	
	// Cache days
	$cachedays = array ( "3" => "3 days",
						 "5" => "5 days",
						 "7" => "7 days",
						 "10" => "10 days",
						 "15" => "15 days",
						 "20" => "20 days",
						 "30" => "30 days",
						 "0" => "Never Expire Thumbnails"
						);
	
	$formElem = new FormElement("setting_cache_days", "Number of Days to Cache Thumbnail");
	$formElem->value = $setting_cache_days;
	$formElem->setTypeAsComboBox($cachedays);
	$formElem->description = "The number of days to hold thumbnails in the cache. Set to a longer time period if website homepages don't change very often";
	$form->addFormElement($formElem);	
	
	// Thumbnail Fetch Method
	$fetchmethod = array( "curl" => "cURL (recommended)",
						  "fopen" => "fopen");
	
	$formElem = new FormElement("setting_fetch_method", "Thumbnail Fetch Method");
	$formElem->value = $setting_fetch_method;
	$formElem->setTypeAsComboBox($fetchmethod);
	$formElem->description = "The type of HTTP call used to fetch thumbnails. fopen is usually less secure and disabled by most web hosts, hence why cURL is recommended.";
	$form->addFormElement($formElem);		
	
	// Show credit link
	$formElem = new FormElement("setting_show_credit", "Show Credit Link");
	$formElem->value = $setting_show_credit;
	$formElem->setTypeAsCheckbox("Creates a link back to WP Portfolio and to DanHarrison.co.uk.");
	$formElem->description = "<strong>I've worked hard on this plugin, please consider keeping the link back to my site!</strong> It's the link back to my site that keeps this plugin free!";
	$form->addFormElement($formElem);	
			
	$form->addButton("clear_thumb_cache", "Clear Thumbnail Cache");
	
	echo $form->toString();
	?>
	
		
	<h2>Uninstalling WP Portfolio</h2>
	<p>If you're going to permanently uninstall WP Portfolio, you can also <a href="admin.php?page=WPP_show_settings&uninstall=yes">remove all settings and data</a>.</p>
		
	<p>&nbsp;</p>	
	<p>&nbsp;</p>
	</div>
	<?php 	
}



/**
 * Function that warns users that the WP Portfolio tag has changed.
 */
function WPPortfolio_tagChangeNotice() {
	global $cache_enabled;
	
	// Only show notice if on one of the plugin pages or any page associated with the WP Portfolio plugin. 
	// Also hide message if requested.
	if (get_option('WPPortfolio_setting_hide_tag_upgrade') != 'true'  
			&& (substr( $_SERVER["PHP_SELF"], -11 ) == 'plugins.php' || WPPortfolio_areWeOnWPPPage())
		) 
	{
		echo '<div class="error"><p><strong>' . 
				sprintf( __('<em>Important!</em> The tag that renders a portfolio within WP Portfolio has now changed. 
							 Use the <a href="%s#tag-upgrade">Old Style Portfolio Tag Upgrader</a> to update all of your tags quickly and easily.
							 <br><br><a href="%s&hidetagupgrade=yes">Hide this message</a>'), 
							WPP_DOCUMENTATION, WPP_SETTINGS) . 
				'</strong></p></div>';
	}
}
add_action( 'admin_notices', 'WPPortfolio_tagChangeNotice' );

/**
 * Function to change all of the old style tags to the new shortcode tag.
 */
function WPPortfolio_changeOldStyleTag() 
{
	?>
	<h2>Old Style Portfolio Code Check</h2>
	<?php
		global $wpdb;
		$table_posts = $wpdb->prefix . 'posts';
		
		// Only bother with published and draft posts and pages, not revisions
		$SQL = "SELECT ID, post_title, post_type, post_status, post_content FROM $table_posts 
				WHERE (post_type = 'post' OR post_type = 'page')
				ORDER BY post_type, post_title, ID
				";
	
		$posts = $wpdb->get_results($SQL, OBJECT);
		$postCount = count($posts);
		
		// In case we take a while, flush results to the page.
		echo "<p>Checking a total of <b>$postCount pages and posts</b> for the old style tag (published and drafts, but not revisions)...</p>";
		flush();
		
		// The URL to edit a post (so that a user can check the change if needed.
		$editurl = get_bloginfo('url') . "/wp-admin/post.php?action=edit&post=";
		
		// The count of posts we've changed
		$foundcount = 0;
				
		echo '<ul style="list-style-type: disc; margin-left: 30px;">';
		
		// Check each posts's content for the old portfolio tag.
		foreach ($posts as $post)
		{
			// Save in variable to allow while loop.
			$postContent = $post->post_content;
			
			// Search for all instances of the old style portfolio string (there may be more than 1 occurence)
			while (preg_match(WPP_STR_SHOW_PORTFOLIO, $postContent, $matches))
			{
				// Escape the old style tag since the HTML comments would not otherwise show up.
				$oldtag = htmlentities($matches[0]);
				
				// Did we find only specific groups?    		
		    	if (isset($matches[2])) { 
					$groups = $matches[2]; 
				} else {
					// Default of showing all groups.
					$groups = false;
				}
				
				// Construct the new tag style
				$newtag = sprintf('[wp-portfolio%s]', ($groups ? ' groups="'.$groups.'"' : ''));
				
				// Replace this instance of the portfolio tag the post content
				$postContent = str_replace($matches[0], $newtag, $postContent);
				
				// Tell user exactly what we changed
				$convertedstr = sprintf('Changed <tt>%s</tt> to new style tag <tt><b>%s</b></tt>.', $oldtag, $newtag);
				
				// Add info about draft posts for extra info for clarity rather than anything else.
				$post_status = '';
				if ($post->post_status == 'draft') {
					$post_status = 'draft ';
				}
				
				echo sprintf('<li>Found in %s%s %s - <a href="%s%s">%s</a> - %s</li>', $post_status, $post->post_type, $post->ID, $editurl, $post->ID, $post->post_title, $convertedstr);
				$foundcount++;
			} 
			
			// Use newly updated $postContent to update the post content in the database
			$SQL = "UPDATE $table_posts SET post_content = '".$wpdb->escape($postContent)."' 
					WHERE ID = $post->ID";
			$wpdb->query($SQL);
			
		}
		echo "</ul>";
		
		// Nice message if no old style tags found
		if ($foundcount > 0) {
			echo "<p><b>All done</b>. A total of <b>$foundcount posts and pages</b> were updated.</p>";
		} else {
			echo "<p><b>All done</b>. No old style tags were found.</p>";
		}
}

/**
 * Show all the documentation in one place.
 */
function WPPortfolio_showDocumentationPage() 
{
	// Check for code upgrade command
	$hidedocs = false;
	if ($_POST && isset($_POST['tag-upgrade'])) {
		$hidedocs = true;
	}
	
	
	?>
	<div class="wrap">
	<div id="icon-options-general" class="icon32">
	<br/>
	</div>
	
	
	<?php if ($hidedocs) : ?>
		<?php WPPortfolio_changeOldStyleTag(); ?>	
	<?php else : ?>
	<h2>WP Portfolio - Documentation</h2>	
	
	<p>All the information you need to run the plugin is available on this page. If you have any comments and feedback on this plugin, 
	please leave comments on the <a href="http://www.danharrison.co.uk/my-wordpress-plugins/wp-portfolio/">WP Portfolio plugin page</a> on my blog.</p>	
	
	<h2>Problems and Support</h2>
	<p>Please check the <a href="http://wordpress.org/extend/plugins/wp-portfolio/faq/">frequently asked questions</a> page if you have any issues. As a last resort, please <a href="http://www.danharrison.co.uk/contact-me">contact me</a> via my website. Since I offer this plugin for free, I <strong>cannot</strong> support issues with the plugin as a top priority. I will always do the best I can to help though.</p>
	
	<h2>Requesting Features</h2>
	<p>My schedule is extremely busy, and so I have little time to add new features to this plugin. If you are keen for a feature to be implemented, I can add new features in return for a small fee which helps 
	cover my time. I'm a freelancer, so my clients are my first priority. By paying a small fee, you effectively become a client, and therefore I can implement desired features more 
	quickly. Please <a href="http://www.danharrison.co.uk/contact-me">contact me</a> via my website if you would like to pay to have a new feature implemented.</p>
	
	<p>If you are prepared to wait, I do welcome ideas, which can be left on the <a href="http://www.danharrison.co.uk/my-wordpress-plugins/wp-portfolio/">WP Portfolio plugin page</a> on 
	my blog. However, please be friendly! I do get far too many nasty messages being left for me (which end up getting deleted).</p>	
	
	<a name="doc-stw"></a>
	<h2>Shrink The Web - Thumbnail Service</h2>
	<p>In order to use this plugin, you'll need to create an account on <a href="http://www.shrinktheweb.com" target="_blank">Shrink The Web (STW)</a>. This is a free service that gives you a number of free thumbnails per month. The WP Portfolio plugin employs the use of caching for efficiency and to reduce the number of times the STW service is accessed. You can check your thumbnail usage on the STW website at any time.</p>
	
	
	<h2>Portfolio Syntax</h2>
	<p>You can use the following syntax for wp-portfolio within any post or page.</p>
	
	<h3>Website Groups</h3>	
	<ul class="wp-group-syntax">
		<li>To show all groups, use <tt><b>[wp-portfolio]</b></tt></li>
		<li>To show just the group with an ID of 1, use <tt><b>[wp-portfolio groups="1"]</b></tt></li>
		<li>To show groups with IDs of 1, 2 and 4, use <tt><b>[wp-portfolio groups="1,2,4"]</b></tt></li>
	</ul>
	
	<h3>Miscellaneous Options</h3>
	<ul class="wp-group-syntax">
		<li>To hide the title/description of all groups shown in a portfolio for just a single post/page without affecting other posts/pages, just use <tt><b>[wp-portfolio hidegroupinfo="1"]</b></tt></li>
	</ul>	
	
	
	
	
	<a name="tag-upgrade"></a>
	<h2>Old Style Portfolio Tag Upgrader</h2>
	<p>As of WP Portfolio V1.09, the tag to trigger the rendering of a portfolio has changed from the comment style tag <tt>&lt;!-- ShowMyPortfolio --&gt;</tt> to the native Wordpress shortcode tag of <tt><b>[wp-portfolio]</b></tt>. 
	The <tt><b>[wp-portfolio]</b></tt> tag is less error-prone, better performing and will also allow me to add new features later on.</p>
	
	<p>To save you time and to help you make the transition to the new style, you can use the tool below to do the conversions for you.</p>
	
	<?php 
	$form = new FormBuilder();
	
	// Use a hidden element to detect if the form has been submitted.
	$formElem = new FormElement("tag-upgrade", false);				
	$formElem->value = true;
	$formElem->setTypeAsHidden();
	$form->addFormElement($formElem);	
	
	$form->setSubmitLabel("Upgrade Portfolio Tags");
	echo $form->toString();
	?>
	
		
	
	<h2>Uninstalling WP Portfolio</h2>
	<p>If you're going to permanently uninstall WP Portfolio, you can also <a href="admin.php?page=WPP_show_settings&uninstall=yes">remove all settings and data</a>.</p>
		
					
	<a name="doc-layout"></a>
	<h2>Portfolio Layout Templates</h2>	
	
	<p>The default templates for the groups and websites below as a reference.</p>
	<ul style="margin-left: 30px; list-style-type: disc;">
		<li><strong><?php echo WPP_STR_GROUP_NAME; ?></strong> - Replace with the group name.</li>
		<li><strong><?php echo WPP_STR_GROUP_DESCRIPTION; ?></strong> - Replace with the group description.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_NAME; ?></strong> - Replace with the website name.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_URL; ?></strong> - Replace with the website url.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_DESCRIPTION; ?></strong> - Replace with the website description.</li>
		<li><strong><?php echo WPP_STR_WEBSITE_THUMBNAIL; ?></strong> - Replace with the website thumbnail including the &lt;img&gt; tag.</li>
	</ul>
	
	<form>
	<table class="form-table">
		<tr class="form-field">
			<th scope="row"><label for="default_template_group">Group Template</label></th>
			<td>
				<textarea name="default_template_group" rows="3"><?php echo htmlentities(WPP_DEFAULT_GROUP_TEMPLATE); ?></textarea>
			</td>
		</tr>		
		<tr class="form-field">
			<th scope="row"><label for="default_template_website">Website Template</label></th>
			<td>
				<textarea name="default_template_website" rows="8"><?php echo htmlentities(WPP_DEFAULT_WEBSITE_TEMPLATE); ?></textarea>
			</td>
		</tr>			
		<tr class="form-field">
			<th scope="row"><label for="default_template_css">Website Template</label></th>
			<td>
				<textarea name="default_template_css" rows="8"><?php echo htmlentities(WPP_DEFAULT_CSS); ?></textarea>
			</td>
		</tr>					
	</table>
	</form>
	<p>&nbsp;</p>
		
	<h2>Showing the Portfolio from PHP</h2>
	<h3>WPPortfolio_getAllPortfolioAsHTML()</h3>
	<p>You can show all or a part of the portfolio from within code by using the <code>WPPortfolio_getAllPortfolioAsHTML($groups, $template_website, $template_group)</code> function.</p>
	
	<p><b>Parameters</b></p>
	<ul class="wp-group-syntax">
		<li><b>$groups</b> - The comma separated list of groups to include. To show all groups, specify <code>false</code> for <code>$groups</code>. (<b>default</b> is <code>false</code>)</li>
		<li><b>$template_website</b> - The HTML template to use for rendering a single website (using the <a href="<?php echo WPP_DOCUMENTATION ?>#doc-layout">template tags above</a>). Specify <code>false</code> to use the website template stored in the settings. (<b>default</b> is <code>false</code>, i.e. use template stored in settings.)</li>
		<li><b>$template_group</b> - The HTML template to use for rendering a group description (using the <a href="<?php echo WPP_DOCUMENTATION ?>#doc-layout">template tags above</a>). Specify <code>false</code> to use the group template stored in the settings. To hide the group description, specify a single space character for <code>$template_group</code>. (<b>default</b> is <code>false</code>, i.e. use template stored in settings.)</li>
	</ul>	
	
	<p>&nbsp;</p>	
	
	<p><b>Example 1 (using website template stored in settings):</b></p> 	
	<pre>
&lt;?php 
if (function_exists('WPPortfolio_getAllPortfolioAsHTML')) {
	echo WPPortfolio_getAllPortfolioAsHTML('1,3');
}
?&gt;
	</pre>
	
	<p><b>Example 2 (with custom templates):</b></p>
	<pre>
&lt;?php 
if (function_exists('WPPortfolio_getAllPortfolioAsHTML'))
{
	$website_template = '
		&lt;div class=&quot;portfolio-website&quot;&gt;
		&lt;div class=&quot;website-thumbnail&quot;&gt;&lt;a href=&quot;%WEBSITE_URL%&quot; target=&quot;_blank&quot;&gt;%WEBSITE_THUMBNAIL%&lt;/a&gt;&lt;/div&gt;
		&lt;div class=&quot;website-name&quot;&gt;&lt;a href=&quot;%WEBSITE_URL%&quot; target=&quot;_blank&quot;&gt;%WEBSITE_NAME%&lt;/a&gt;&lt;/div&gt;
		&lt;div class=&quot;website-description&quot;&gt;%WEBSITE_DESCRIPTION%&lt;/div&gt;
		&lt;div class=&quot;website-clear&quot;&gt;&lt;/div&gt;
		&lt;/div&gt;';
		
	$group_template = '
		&lt;h2&gt;%GROUP_NAME%&lt;/h2&gt;
		&lt;p&gt;%GROUP_DESCRIPTION%&lt;/p&gt;';	
	
	echo WPPortfolio_getAllPortfolioAsHTML('1,2', $website_template, $group_template);
}
?&gt;
	</pre>		
		
	<p>&nbsp;</p>		
	
	<h3>WPPortfolio_getRandomPortfolioSelectionAsHTML()</h3>
	<p>You can show a random selection of your portfolio from within code by using the <code>WPPortfolio_getRandomPortfolioSelectionAsHTML($groups, $count, $template_website)</code> function. Please note that there is no group information shown when this function is used.</p>
	
	<p><b>Parameters</b></p>
	<ul class="wp-group-syntax">
		<li><b>$groups</b> - The comma separated list of groups to make a random selection from. To choose from all groups, specify <code>false</code> for <code>$groups</code> (<b>default</b> is <code>false</code>).</li>
		<li><b>$count</b> - The number of websites to show in the random selection. (<b>default</b> is <code>3</code>)</li>
		<li><b>$template_website</b> - The HTML template to use for rendering a single website (using the <a href="<?php echo WPP_DOCUMENTATION ?>#doc-layout">template tags above</a>). Specify <code>false</code> to use the website template stored in the settings. (<b>default</b> is <code>false</code>, i.e. use template stored in settings.)</li>
	</ul>
	
	<p>&nbsp;</p>	
	
	<p><b>Example 1 (using website template stored in settings):</b></p> 	
	<pre>
&lt;?php 
if (function_exists('WPPortfolio_getRandomPortfolioSelectionAsHTML')) {
	echo WPPortfolio_getRandomPortfolioSelectionAsHTML('1,4', 4);
}
?&gt;
	</pre>
	
	<p><b>Example 2 (with custom template):</b></p>
	<pre>
&lt;?php 
if (function_exists('WPPortfolio_getRandomPortfolioSelectionAsHTML')) {
	$website_template = '
		&lt;div class=&quot;portfolio-website&quot;&gt;
		&lt;div class=&quot;website-thumbnail&quot;&gt;&lt;a href=&quot;%WEBSITE_URL%&quot; target=&quot;_blank&quot;&gt;%WEBSITE_THUMBNAIL%&lt;/a&gt;&lt;/div&gt;
		&lt;div class=&quot;website-name&quot;&gt;&lt;a href=&quot;%WEBSITE_URL%&quot; target=&quot;_blank&quot;&gt;%WEBSITE_NAME%&lt;/a&gt;&lt;/div&gt;
		&lt;div class=&quot;website-clear&quot;&gt;&lt;/div&gt;
		&lt;/div&gt;';
	echo WPPortfolio_getRandomPortfolioSelectionAsHTML('1,4', 4, $website_template);
}
?&gt;
	</pre>
		
	<?php endif; ?>
	<p>&nbsp;</p>	
	<p>&nbsp;</p>
</div>
	
	<?php
}

/**
 * Show only the settings relating to layout of the portfolio.
 */
function WPPortfolio_showLayoutSettingsPage() 
{
?>
	<div class="wrap">
	<div id="icon-themes" class="icon32">
	<br/>
	</div>
	<h2>WP Portfolio - Layout Settings</h2>
<?php 	

	// Get all the options from the database for the form
	$setting_template_website = stripslashes(get_option('WPPortfolio_setting_template_website'));
	$setting_template_group   = stripslashes(get_option('WPPortfolio_setting_template_group'));	
	$setting_template_css     = stripslashes(get_option('WPPortfolio_setting_template_css'));
	
	
	// If we don't have the version in the settings, we're not installed
	if (!get_option('WPPortfolio_version')) {
		WPPortfolio_showMessage("No WP Portfolio settings were found, so it appears that the plugin has been uninstalled. Please <b>deactivate</b> and then <b>activate</b> the WP Portfolio plugin again to fix this.", true);
		return false;
	}
	
			
	// Check if updated data.
	if ( isset($_POST) && isset($_POST['update']) )
	{
		$setting_template_website = trim($_POST['setting_template_website']);
		$setting_template_group   = trim($_POST['setting_template_group']);
		$setting_template_css     = trim($_POST['setting_template_css']);
		
		update_option('WPPortfolio_setting_template_website', $setting_template_website);
		update_option('WPPortfolio_setting_template_group',   $setting_template_group);			
		update_option('WPPortfolio_setting_template_css',     $setting_template_css);
				
		WPPortfolio_showMessage();		
	}	
	
	
	$form = new FormBuilder();	
	
	$formElem = new FormElement("setting_template_website", "Website HTML Template");				
	$formElem->value = htmlentities(stripslashes($setting_template_website));
	$formElem->description = '&bull; This is the template used to render each of the websites.<br/>
							  &bull; A complete list of tags is available in the <a href="'.WPP_DOCUMENTATION.'#doc-layout">Portfolio Layout Templates</a> section in the documentation.';
	$formElem->setTypeAsTextArea(8, 70); 
	$form->addFormElement($formElem);
	// TODO class="form-field"
	
	$formElem = new FormElement("setting_template_group", "Group HTML Template");				
	$formElem->value = htmlentities(stripslashes($setting_template_group));
	$formElem->description = '&bull; This is the template used to render each of the groups that the websites belong to.<br/>
							  &bull; A complete list of tags is available in the <a href="'.WPP_DOCUMENTATION.'#doc-layout">Portfolio Layout Templates</a> section in the documentation.';
	$formElem->setTypeAsTextArea(3, 70); 
	$form->addFormElement($formElem);	
	
	$formElem = new FormElement("setting_template_css", "Template CSS");				
	$formElem->value = htmlentities(stripslashes($setting_template_css));
	$formElem->description = '&bull; This is the CSS code used to style the portfolio.<br/>
							  &bull; <strong>Advanced Tip:</strong> Once you\'re happy with the style, you should really move the CSS into your template <tt>style.css</tt>. This is so that visitor browsers can cache the stylesheet and reduce loading times. Any CSS placed here will 
							  be injected into the template &lt;head&gt; tag, which is not the most efficient method of delivering CSS.<br/>
							  &bull; A complete list of tags is available in the <a href="'.WPP_DOCUMENTATION.'#doc-layout">Portfolio Layout Templates</a> section in the documentation.';
	$formElem->setTypeAsTextArea(10, 70); 
	$form->addFormElement($formElem);	
	
	
	echo $form->toString();
	
	?>	

	

</div>
<?php 
}

/**
 * Page that shows a list of websites in your portfolio.
 */
function WPPortfolio_show_websites()
{
?>
<div class="wrap">
	<div id="icon-themes" class="icon32">
	<br/>
	</div>
	<h2>Summary of Websites in your Portfolio</h2>
	<br>
<?php 		

    // See if a group parameter was specified, if so, use that to show websites
    // in just that group
    $groupid = false;
    if (isset($_GET['groupid'])) {
    	$groupid = $_GET['groupid'] + 0;
    }
    
	$siteid = 0;
	if (isset($_GET['siteid'])) {
		$siteid = (is_numeric($_GET['siteid']) ? $_GET['siteid'] + 0 : 0);
	}	    

	global $wpdb;
	$websites_table = $wpdb->prefix . TABLE_WEBSITES;
	$groups_table   = $wpdb->prefix . TABLE_WEBSITE_GROUPS;

	
	// ### DELETE Check if we're deleting a website
	if ($siteid > 0 && isset($_GET['delete']))
	{
		$websitedetails = WPPortfolio_getWebsiteDetails($siteid);
		
		if (isset($_GET['confirm']))
		{
			$delete_website = "DELETE FROM $websites_table WHERE siteid = '".$wpdb->escape($siteid)."' LIMIT 1";
			if ($wpdb->query( $delete_website )) {
				WPPortfolio_showMessage("Website was successfully deleted.");
			}
			else {
				WPPortfolio_showMessage("Sorry, but an unknown error occured whist trying to delete the selected website from the portfolio.", true);
			}
		}
		else
		{
			$message = 'Are you sure you want to delete "'.$websitedetails['sitename'].'" from your portfolio?<br/><br/><a href="'.WPP_WEBSITE_SUMMARY.'&delete=yes&confirm=yes&siteid='.$websitedetails['siteid'].'">Yes, delete.</a> &nbsp; <a href="'.WPP_WEBSITE_SUMMARY.'">NO!</a>';
			WPPortfolio_showMessage($message);
			return;
		}
	}		
	

	// Determine if showing only 1 group
	$WHERE_CLAUSE = false;
	if ($groupid > 0) {
		$WHERE_CLAUSE = "WHERE $groups_table.groupid = '$groupid'";
	}
	
	// Get website details, merge with group details
	$SQL = "SELECT * FROM $websites_table
			LEFT JOIN $groups_table ON $websites_table.sitegroup = $groups_table.groupid
			$WHERE_CLAUSE
	 		ORDER BY $websites_table.sitename
	 		";	
		
	// DEBUG - Uncomment if needed
	//$wpdb->show_errors();
	$websites = $wpdb->get_results($SQL, OBJECT);	
			
	// Only show table if there are websites to show
	if ($websites)
	{
			// If just showing 1 group
		if ($groupid) {
			echo '<h3>Showing websites in the \''.$websites[0]->groupname.'\' group (<a href="'.WPP_WEBSITE_SUMMARY.'" class="showall">or Show All</a>)</h3>'; 
		}		
		
		$table = new TableBuilder();
		$table->attributes = array("id" => "wpptable");

		$column = new TableColumn("ID", "id");
		$column->cellClass = "wpp-id";
		$table->addColumn($column);
		
		$column = new TableColumn("Thumbnail", "thumbnail");
		$column->cellClass = "wpp-thumbnail";
		$table->addColumn($column);
		
		$column = new TableColumn("Site Name", "sitename");
		$column->cellClass = "wpp-name";
		$table->addColumn($column);
		
		$column = new TableColumn("URL", "siteurl");
		$column->cellClass = "wpp-url";
		$table->addColumn($column);

		$column = new TableColumn("Custom Thumbnail?", "customthumb");
		$column->cellClass = "wpp-customurl";
		$table->addColumn($column);						
		
		$column = new TableColumn("Group", "group");
		$column->cellClass = "wpp-small";
		$table->addColumn($column);
					
		$column = new TableColumn("Action", "action");
		$column->cellClass = "wpp-small action-links";
		$column->headerClass = "action-links";		
		$table->addColumn($column);							
			
		foreach ($websites as $websitedetails)
		{
			// First part of a link to visit a website
			$websiteClickable = '<a href="'.$websitedetails->siteurl.'" target="_new" title="Visit the website \''.stripslashes($websitedetails->sitename).'\'">';
			$editClickable    = '<a href="'.WPP_MODIFY_WEBSITE.'&editmode=edit&siteid='.$websitedetails->siteid.'" title="Edit \''.stripslashes($websitedetails->sitename).'\'">';
			
			$rowdata = array();
			$rowdata['id'] 			= $websitedetails->siteid;			
			
			$rowdata['sitename'] 	= stripslashes($websitedetails->sitename);			
			$rowdata['siteurl'] 	= $websiteClickable.$websitedetails->siteurl.'</a>';
			
			// Custom URL will typically not be specified, so show n/a for clarity.
			if ($websitedetails->customthumb)
			{
				// Use custom thumbnail rather than screenshot
				$rowdata['thumbnail'] 	= '<img src="'.WPPortfolio_getAdjustedCustomThumbnail($websitedetails->customthumb, "sm").'" />';
				
				$rowdata['customthumb'] = 'Yes - <a href="'.$websitedetails->customthumb.'" target="_new" title="Open custom thumbnail in a new window">View Image</a>';
			} 
			// Not using custom thumbnail
			else 
			{
				$rowdata['thumbnail'] 	= '<img src="'.WPPortfolio_getThumbnail($websitedetails->siteurl, "sm").'" width="120" height="90"  />';
				$rowdata['customthumb'] = '-';
			}
			
			$rowdata['group'] 		= '<a href="'.WPP_WEBSITE_SUMMARY.'&groupid='.$websitedetails->groupid.'" title="Show websites only in the \''.stripslashes($websitedetails->groupname).'\' group">'.stripslashes($websitedetails->groupname).'</a>';
			$rowdata['action'] 		= '<a href="'.WPP_WEBSITE_SUMMARY.'&delete=yes&siteid='.$websitedetails->siteid.'">Delete</a>&nbsp;|&nbsp;' .
									  $editClickable.'Edit</a>';
		
			$table->addRow($rowdata);
		}
		
		// Finally show table
		echo $table->toString();
		
	} // end of if websites
	else {
		WPPortfolio_showMessage("There are currently no websites in the portfolio.", true);
	}
	
	?>	
</div>
<?php 
	
}



/**
 * Shows the page listing the available groups.
 */
function WPPortfolio_show_website_groups()
{
?>
<div class="wrap">
	<div id="icon-edit" class="icon32">
	<br/>
	</div>
	<h2>Website Groups</h2>
	<br/>

	<?php 
	global $wpdb;
	$groups_table = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	$websites_table = $wpdb->prefix . TABLE_WEBSITES;
	
    // Get group ID
    $groupid = false;
    if (isset($_GET['groupid'])) {
    	$groupid = $_GET['groupid'] + 0;
    }	
	
	// ### DELETE ### Check if we're deleting a group
	if ($groupid > 0 && isset($_GET['delete'])) 
	{				
		// Now check that ID actually relates to a real group
		$groupdetails = WPPortfolio_getGroupDetails($groupid);
		
		// If group doesn't really exist, then stop.
		if (count($groupdetails) == 0) {
			WPPortfolio_showMessage('Sorry, but no group with that ID could be found. Please click <a href="'.WPP_GROUP_SUMMARY.'">here</a> to return to the list of groups.', true);
			return;
		}
		
		// Count the number of websites in this group and how many groups exist
		$website_count = $wpdb->get_var("SELECT COUNT(*) FROM $websites_table WHERE sitegroup = '".$wpdb->escape($groupdetails['groupid'])."'");
		$group_count   = $wpdb->get_var("SELECT COUNT(*) FROM $groups_table");
		
		$groupname = stripcslashes($groupdetails['groupname']);
		
		// Check that group doesn't have a load of websites assigned to it.
		if ($website_count > 0)  {
			WPPortfolio_showMessage("Sorry, the group '$groupname' still contains <b>$website_count</b> websites. Please ensure the group is empty before deleting it.");
			return;
		}
		
		// If we're deleting the last group, don't let it happen
		if ($group_count == 1)  {
			WPPortfolio_showMessage("Sorry, but there needs to be at least 1 group in the portfolio. Please add a new group before deleting '$groupname'.");
			return;
		}
		
		// OK, got this far, confirm we want to delete.
		if (isset($_GET['confirm']))
		{
			$delete_group = "DELETE FROM $groups_table WHERE groupid = '".$wpdb->escape($groupid)."' LIMIT 1";
			if ($wpdb->query( $delete_group )) {
				WPPortfolio_showMessage("Group was successfully deleted.");
			}
			else {
				WPPortfolio_showMessage("Sorry, but an unknown error occured whist trying to delete the selected group from the portfolio.", true);
			}
		}
		else
		{
			$message = 'Are you sure you want to delete the group "'.$groupname.'" from your portfolio?<br/><br/><a href="'.WPP_GROUP_SUMMARY.'&delete=yes&confirm=yes&groupid='.$groupid.'">Yes, delete.</a> &nbsp; <a href="'.WPP_GROUP_SUMMARY.'">NO!</a>';
			WPPortfolio_showMessage($message);
			return;
		}
	}	
	
	
	
	// Get website details, merge with group details
	$SQL = "SELECT * FROM $groups_table
	 		ORDER BY grouporder, groupname";	
	
	// DEBUG Uncomment if needed
	// $wpdb->show_errors();
	$groups = $wpdb->get_results($SQL, OBJECT);
		
	
	// Only show table if there are any results.
	if ($groups)
	{					
		$table = new TableBuilder();
		$table->attributes = array("id" => "wpptable");

		$column = new TableColumn("ID", "id");
		$column->cellClass = "wpp-id";
		$table->addColumn($column);		
		
		$column = new TableColumn("Name", "name");
		$column->cellClass = "wpp-name";
		$table->addColumn($column);	

		$column = new TableColumn("Description", "description");
		$table->addColumn($column);	

		$column = new TableColumn("# Websites", "websitecount");
		$column->cellClass = "wpp-small wpp-center";
		$table->addColumn($column);			
		
		$column = new TableColumn("Ordering", "ordering");
		$column->cellClass = "wpp-small wpp-center";
		$table->addColumn($column);		
		
		$column = new TableColumn("Action", "action");
		$column->cellClass = "wpp-small action-links";
		$column->headerClass = "action-links";
		$table->addColumn($column);		
		
		?>
		<p>The websites will be rendered in groups in the order shown in the table.</p>
		<?php 
		
		foreach ($groups as $groupdetails) 
		{
			$groupClickable = '<a href="'.WPP_WEBSITE_SUMMARY.'&groupid='.$groupdetails->groupid.'" title="Show websites only in the \''.$groupdetails->groupname.'\' group">';
			
			// Count websites in this group
			$website_count = $wpdb->get_var("SELECT COUNT(*) FROM $websites_table WHERE sitegroup = '".$wpdb->escape($groupdetails->groupid)."'");
			
			$rowdata = array();
			
			$rowdata['id']			 	= $groupdetails->groupid;
			$rowdata['name']		 	= $groupClickable.stripslashes($groupdetails->groupname).'</a>';
			$rowdata['description']	 	= stripslashes($groupdetails->groupdescription);
			$rowdata['websitecount'] 	= $groupClickable.$website_count.($website_count == 1 ? ' website' : ' websites')."</a>";
			$rowdata['ordering']	 	= $groupdetails->grouporder;
			$rowdata['action']		 	= '<a href="'.WPP_GROUP_SUMMARY.'&delete=yes&groupid='.$groupdetails->groupid.'">Delete</a>&nbsp;|&nbsp;' .
										  '<a href="'.WPP_MODIFY_GROUP.'&editmode=edit&groupid='.$groupdetails->groupid.'">Edit</a></td>';
			
			$table->addRow($rowdata);
		}
		
		
		// Finally show table
		echo $table->toString();		
		
	} // end of if groups
	
	// No groups to show
	else {
		WPPortfolio_showMessage("There are currently no groups in the portfolio.", true);
	}
	?>
</div>
<?php 
	
}


/**
 * Shows the page that allows the details of a website to be modified or added to the portfolio.
 */
function WPPortfolio_modify_website()
{
	// Determine if we're in edit mode. Ensure we get correct mode regardless of where it is.
	$editmode = false;
	if (isset($_POST['editmode'])) {
		$editmode = ($_POST['editmode'] == 'edit');
	} else if (isset($_GET['editmode'])) {
		$editmode = ($_GET['editmode'] == 'edit');
	}	
	
	// Get the site ID. Ensure we get ID regardless of where it is.
	$siteid = 0;
	if (isset($_POST['website_siteid'])) {
		$siteid = (is_numeric($_POST['website_siteid']) ? $_POST['website_siteid'] + 0 : 0);
	} else if (isset($_GET['siteid'])) {
		$siteid = (is_numeric($_GET['siteid']) ? $_GET['siteid'] + 0 : 0);
	}	
	
	// Work out page heading
	$verb = "Add New";
	if ($editmode) { 
		$verb = "Modify";
	}
	
	?>
	<div class="wrap">
	<div id="icon-themes" class="icon32">
	<br/>
	</div>
	<h2><?php echo $verb; ?> Website Details</h2>	
	<?php 	
		
	
	// Check id is a valid number if editing $editmode
	if ($editmode && $siteid == 0) {
		WPPortfolio_showMessage('Sorry, but no website with that ID could be found. Please click <a href="'.WPP_WEBSITE_SUMMARY.'">here</a> to return to the list of websites.', true);
		return;
	}	
	

	// Check if we're adding or editing a website
	if ($editmode && $siteid > 0)
	{
		// Get details from the database
		$websitedetails = WPPortfolio_getWebsiteDetails($siteid);

		// False alarm, couldn't find it.
		if (count($websitedetails) == 0) {
			$editmode = false;
		}		
	} // end of editing check
	
	
	// Check if website is being added, if so, add to the database.
	if ( isset($_POST) && isset($_POST['update']) )
	{
		// Grab specified details
		$data = array();
		$data['siteid'] 			= $_POST['website_siteid'];
		$data['sitename'] 			= trim(strip_tags($_POST['website_sitename']));
		$data['siteurl'] 			= trim(strip_tags($_POST['website_siteurl']));
		$data['sitedescription'] 	= $_POST['website_sitedescription'];
		$data['sitegroup'] 			= $_POST['website_sitegroup'];
		$data['customthumb']		= trim(strip_tags($_POST['website_customthumb']));
		
		
		// Keep track of errors for validation
		$errors = array();
				
		// Ensure all fields have been completed
		if (!($data['sitename'] && $data['siteurl'] && $data['sitedescription'])) {
			array_push($errors, "Please check that you have completed the site name, url and description fields.");
		}	
		
		// Continue if there are no errors
		if (count($errors) == 0)
		{
			global $wpdb;
			$table_name = $wpdb->prefix . TABLE_WEBSITES;
			
			// Change query based on add or edit
			if ($editmode) {							
				$query = arrayToSQLUpdate($table_name, $data, 'siteid');
			}

			// Add
			else {
				unset($data['siteid']); // Don't need id for an insert	
				$query = arrayToSQLInsert($table_name, $data);	
			}			
						
			// Try to put the data into the database
			$wpdb->show_errors();
			$wpdb->query($query);
			
			// When adding, clean fields so that we don't show them again.
			if ($editmode) {
				WPPortfolio_showMessage("Website details successfully updated.");
				
				// Retrieve the details from the database again
				$websitedetails = WPPortfolio_getWebsiteDetails($siteid);				
			} 
			// When adding, empty the form again
			else
			{	
				WPPortfolio_showMessage("Website details successfully added.");
					
				$data['siteid'] 			= false;
				$data['sitename'] 			= false;
				$data['siteurl'] 			= false;
				$data['sitedescription'] 	= false;
				$data['sitegroup'] 			= false;
				$data['customthumb']		= false;
			}
								
		} // end of error checking
	
		// Handle error messages
		else
		{
			$message = "Sorry, but unfortunately there were some errors. Please fix the errors and try again.<br><br>";
			$message .= "<ul style=\"margin-left: 20px; list-style-type: square;\">";
			
			// Loop through all errors in the $error list
			foreach ($errors as $errormsg) {
				$message .= "<li>$errormsg</li>";
			}
						
			$message .= "</ul>";
			WPPortfolio_showMessage($message, true);
			$websitedetails = data;
		}
	}
		
	$form = new FormBuilder();
		
	$formElem = new FormElement("website_sitename", "Website Name");				
	$formElem->value = $websitedetails['sitename'];
	$formElem->description = "The proper name of the website.";
	$form->addFormElement($formElem);
	
	$formElem = new FormElement("website_siteurl", "Website URL");				
	$formElem->value = $websitedetails['siteurl'];
	$formElem->description = "The URL for the website, including the leading <em>http://</em>.";
	$form->addFormElement($formElem);	
	
	$formElem = new FormElement("website_sitedescription", "Website Description");				
	$formElem->value = $websitedetails['sitedescription'];
	$formElem->description = "The description of your website. HTML is permitted.";
	$formElem->setTypeAsTextArea(4, 70);
	$form->addFormElement($formElem);	
	
	global $wpdb;
	$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	$SQL = "SELECT * FROM $table_name ORDER BY groupname";	
	$groups = $wpdb->get_results($SQL, OBJECT);	
	$grouplist = array();
	
	foreach ($groups as $group) {
		$grouplist[$group->groupid] =  stripslashes($group->groupname);
	}	
		
	$formElem = new FormElement("website_sitegroup", "Website Group");
	$formElem->setTypeAsComboBox($grouplist);				
	$formElem->value = $websitedetails['sitegroup'];
	$formElem->description = "The group you want to assign this website to.";
	$form->addFormElement($formElem);	
	
	$formElem = new FormElement("website_customthumb", "Custom Thumbnail URL");				
	$formElem->value = $websitedetails['customthumb'];
	$formElem->cssclass = "long-text";
	$formElem->description = "If specified, the URL of a custom thumbnail to use <em>instead</em> of the screenshot of the URL above. 							  
							  <br/>&bull; The image URL must include the leading <em>http://</em>. e.g. <em>http://www.yoursite.com/wp-content/uploads/yourfile.jpg</em>							  
							  <br/>&bull; Leave this field blank to use an automatically generated screenshot of the website specified above.
							  <br/>&bull; Custom thumbnails are automatically resized to match the size of the other thumbnails. 
							  ";
	$form->addFormElement($formElem);		
	
	// Hidden Elements
	$formElem = new FormElement("website_siteid", false);				
	$formElem->value = $websitedetails['siteid'];
	$formElem->setTypeAsHidden();
	$form->addFormElement($formElem);
	
	$formElem = new FormElement("editmode", false);				
	$formElem->value = ($editmode ? "edit" : "add");
	$formElem->setTypeAsHidden();
	$form->addFormElement($formElem);	
	
	
	$form->setSubmitLabel(($editmode ? "Update" : "Add"). " Website Details");		
	echo $form->toString();
			
	?>	
	<br><br>
	</div><!-- wrap -->
	<?php 	
}


/**
 * Shows the page that allows a group to be modified.
 */
function WPPortfolio_modify_group()
{
	// Determine if we're in edit mode. Ensure we get correct mode regardless of where it is.
	$editmode = false;
	if (isset($_POST['editmode'])) {
		$editmode = ($_POST['editmode'] == 'edit');
	} else if (isset($_GET['editmode'])) {
		$editmode = ($_GET['editmode'] == 'edit');
	}	
	
	// Get the Group ID. Ensure we get ID regardless of where it is.
	$groupid = 0;
	if (isset($_POST['group_groupid'])) {
		$groupid = (is_numeric($_POST['group_groupid']) ? $_POST['group_groupid'] + 0 : 0);
	} else if (isset($_GET['groupid'])) {
		$groupid = (is_numeric($_GET['groupid']) ? $_GET['groupid'] + 0 : 0);
	}

	$verb = "Add New";
	if ($editmode) {
		$verb = "Modify";
	}
	
	// Show title to determine action
	?>
	<div class="wrap">
	<div id="icon-edit" class="icon32">
	<br/>
	</div>
	<h2><?php echo $verb; ?> Group Details</h2>
	<?php 
	
	// Check id is a valid number if editing $editmode
	if ($editmode && $groupid == 0) {
		WPPortfolio_showMessage('Sorry, but no group with that ID could be found. Please click <a href="'.WPP_GROUP_SUMMARY.'">here</a> to return to the list of groups.', true);
		return;
	}	

	// ### EDIT ### Check if we're adding or editing a group
	if ($editmode && $groupid > 0)
	{
		// Get details from the database				
		$groupdetails = WPPortfolio_getGroupDetails($groupid);

		// False alarm, couldn't find it.
		if (count($groupdetails) == 0) {
			$editmode = false;
		}
		
	} // end of editing check
			
	// Check if group is being updated/added.
	if ( isset($_POST) && isset($_POST['update']) )
	{
		// Grab specified details
		$data = array();
		$data['groupid'] 			= $groupid;	
		$data['groupname'] 		  	= strip_tags($_POST['group_groupname']);
		$data['groupdescription'] 	= $_POST['group_groupdescription'];
		$data['grouporder'] 		= $_POST['group_grouporder'] + 0; // Add zero to convert to number
						
		// Keep track of errors for validation
		$errors = array();
				
		// Ensure all fields have been completed
		if (!($data['groupname'] && $data['groupdescription'])) {
			array_push($errors, "Please check that you have completed the group name and description fields.");
		}	
		
		// Continue if there are no errors
		if (count($errors) == 0)
		{
			global $wpdb;
			$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;

			// Change query based on add or edit
			if ($editmode) {							
				$query = arrayToSQLUpdate($table_name, $data, 'groupid');
			}

			// Add
			else {
				unset($data['groupid']); // Don't need id for an insert	
				$query = arrayToSQLInsert($table_name, $data);	
			}
			
			// Try to put the data into the database
			$wpdb->show_errors();
			$wpdb->query($query);
			
			// When editing, show what we've just been editing.
			if ($editmode) {
				WPPortfolio_showMessage("Group details successfully updated.");
				
				// Retrieve the details from the database again
				$groupdetails = WPPortfolio_getGroupDetails($groupid);
			} 
			// When adding, empty the form again
			else {																							
				WPPortfolio_showMessage("Group details successfully added.");
				
				$groupdetails['groupid'] 			= false;
				$groupdetails['groupname'] 			= false;
				$groupdetails['groupdescription'] 	= false;
				$groupdetails['grouporder'] 		= false;
			}

		} // end of error checking
	
		// Handle error messages
		else
		{
			$message = "Sorry, but unfortunately there were some errors. Please fix the errors and try again.<br><br>";
			$message .= "<ul style=\"margin-left: 20px; list-style-type: square;\">";
			
			// Loop through all errors in the $error list
			foreach ($errors as $errormsg) {
				$message .= "<li>$errormsg</li>";
			}
						
			$message .= "</ul>";
			WPPortfolio_showMessage($message, true);
			$groupdetails = $data;
		}
	}
	
	$form = new FormBuilder();
	
	$formElem = new FormElement("group_groupname", "Group Name");				
	$formElem->value = $groupdetails['groupname'];
	$formElem->description = "The name for this group of websites.";
	$form->addFormElement($formElem);	
	
	$formElem = new FormElement("group_groupdescription", "Group Description");				
	$formElem->value = $groupdetails['groupdescription'];
	$formElem->description = "The description of your group. HTML is permitted.";
	$formElem->setTypeAsTextArea(4, 70);
	$form->addFormElement($formElem);		
	
	$formElem = new FormElement("group_grouporder", "Group Order");				
	$formElem->value = $groupdetails['grouporder'];
	$formElem->description = "The number to use for ordering the groups. Groups are rendered in ascending order, first by this order value (lowest value first), then by group name. 
				e.g. Groups (A, B, C, D) with ordering (50, 100, 0, 50) will be rendered as (C, A, D, B).  
				If all groups have 0 for ordering, then the groups are rendered in alphabetical order. ";
	$form->addFormElement($formElem);		
	
	// Hidden Elements
	$formElem = new FormElement("group_groupid", false);				
	$formElem->value = $groupdetails['groupid'];
	$formElem->setTypeAsHidden();
	$form->addFormElement($formElem);
	
	$formElem = new FormElement("editmode", false);				
	$formElem->value = ($editmode ? "edit" : "add");
	$formElem->setTypeAsHidden();
	$form->addFormElement($formElem);	
	
	
	$form->setSubmitLabel(($editmode ? "Update" : "Add"). " Group Details");		
	echo $form->toString();	
		
	?>		
	<br><br>
	</div><!-- wrap -->
	<?php 	
}


/**
 * Function: WPPortfolio_()
 * Install the WP Portfolio plugin, initialise the default settings, and create the tables for the websites and groups.
 */
function WPPortfolio_install()
{
	global $wpdb;

	// ### Create Default Settings
	if (!get_option('WPPortfolio_setting_stw_access_key'))
		update_option('WPPortfolio_setting_stw_access_key', "");
	
	if (!get_option('WPPortfolio_setting_stw_secret_key'))
		update_option('WPPortfolio_setting_stw_secret_key', "");
	
	if (!get_option('WPPortfolio_setting_stw_thumb_size'))
		update_option('WPPortfolio_setting_stw_thumb_size', "lg");
				
	if (!get_option('WPPortfolio_setting_cache_days'))
		update_option('WPPortfolio_setting_cache_days', 7);

	if (!get_option('WPPortfolio_setting_template_website'))
		update_option('WPPortfolio_setting_template_website', WPP_DEFAULT_WEBSITE_TEMPLATE);		
		
	if (!get_option('WPPortfolio_setting_template_group'))
		update_option('WPPortfolio_setting_template_group', WPP_DEFAULT_GROUP_TEMPLATE);

	if (!get_option('WPPortfolio_setting_template_css'))
		update_option('WPPortfolio_setting_template_css', WPP_DEFAULT_CSS);	
	
	if (!get_option('WPPortfolio_setting_fetch_method'))
		update_option('WPPortfolio_setting_fetch_method', "curl");	
		
		if (!get_option('WPPortfolio_setting_show_credit'))
		update_option('WPPortfolio_setting_show_credit', "on");
			
		
	// Check the current version of the database
	$installed_ver  = get_option("WPPortfolio_version") + 0;
	$current_ver    = WPP_VERSION + 0;
	$upgrade_tables = ($current_ver > $installed_ver);
	
			
	// Table names
	$table_websites	= $wpdb->prefix . TABLE_WEBSITES;
	$table_groups 	= $wpdb->prefix . TABLE_WEBSITE_GROUPS;
				
	// Check tables exist
	$table_websites_exists	= ($wpdb->get_var("SHOW TABLES LIKE '$table_websites'") == $table_websites);
	$table_groups_exists	= ($wpdb->get_var("SHOW TABLES LIKE '$table_groups'") == $table_groups);
	
			
	$wpdb->show_errors();

	// #### Create Tables - Websites
	if (!$table_websites_exists || $upgrade_tables) 
	{
		$sql = "CREATE TABLE `$table_websites` (
  				   `siteid` int(10) unsigned NOT NULL auto_increment,
				   `sitename` varchar(150),
				   `siteurl` varchar(255),
				   `sitedescription` TEXT,
				   `sitegroup` int(10) unsigned NOT NULL,
				   `customthumb` varchar(255),
				   PRIMARY KEY (`siteid`)
				) ";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
	}
	
	// #### Create Tables - Groups
	if (!$table_groups_exists || $upgrade_tables)
	{
		$sql = "CREATE TABLE `$table_groups` (
  				   `groupid` int(10) UNSIGNED NOT NULL auto_increment,
				   `groupname` varchar(150),
				   `groupdescription` TEXT,
				   `grouporder` INT(1) UNSIGNED NOT NULL DEFAULT '0',
				   PRIMARY KEY (`groupid`)
				) ";
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);
		
		// Creating new table? Add default group that has ID of 0
		if (!$table_groups_exists)
		{
			$SQL = "INSERT INTO `$table_groups` (groupid, groupname, groupdescription) VALUES (1, 'My Websites', 'These are all my websites.')";
	 		$results = $wpdb->query($SQL);
		}
	}	
		
	// Update the version regardless
	update_option("WPPortfolio_version", WPP_VERSION);
	
	// Create cache directory
	WPPortfolio_createCacheDirectory();
}
register_activation_hook(__FILE__,'WPPortfolio_install');


/**
 * Add the custom stylesheet for this plugin.
 */
function WPPortfolio_stylesheet()
{
	// Only show our stylesheet on a WP Portfolio page to avoid breaking other plugins.
	if (WPPortfolio_areWeOnWPPPage()) {
    	$url = WP_PLUGIN_URL . '/wp-portfolio/portfolio.css';
    	echo "\n".'<link rel="stylesheet" type="text/css" href="' . $url . '" />'."\n";
	}
}
add_action('admin_head', 'WPPortfolio_stylesheet');


/**
 * Method called when we want to uninstall the portfolio plugin to remove the database tables.
 */
function WPPortfolio_uninstall() 
{
	// Remove all options from the database
	delete_option('WPPortfolio_setting_stw_access_key');
	delete_option('WPPortfolio_setting_stw_secret_key');	
	delete_option('WPPortfolio_setting_stw_thumb_size');
	delete_option('WPPortfolio_setting_cache_days');
	
	delete_option('WPPortfolio_setting_template_website');
	delete_option('WPPortfolio_setting_template_group');
	delete_option('WPPortfolio_setting_template_css');
			
	delete_option('WPPortfolio_version');
		
	
	// Remove all tables for the portfolio
	global $wpdb;
	$table_name    = $wpdb->prefix . TABLE_WEBSITES;
	$uninstall_sql = "DROP TABLE IF EXISTS ".$table_name;
	$wpdb->query($uninstall_sql);
	
	$table_name    = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	$uninstall_sql = "DROP TABLE IF EXISTS ".$table_name;
	$wpdb->query($uninstall_sql);
	
	// Remove cache
	$actualThumbPath = WPPortfolio_getThumbPathActualDir();
	WPPortfolio_unlinkRecursive($actualThumbPath);		
		
	WPPortfolio_showMessage("Deleted WP Portfolio database entries.");
}




/**
 * This method is called just before the <head> tag is closed. We inject our custom CSS into the 
 * webpage here.
 */
function WPPortfolio_renderCSS() 
{
	// Only render CSS if there's some to show.
	$setting_template_css = trim(stripslashes(get_option('WPPortfolio_setting_template_css')));
	if ($setting_template_css) {
		echo "\n<!-- WP Portfolio Stylesheet -->\n";
		echo "<style type=\"text/css\">\n$setting_template_css\n</style>";
		echo "\n<!-- WP Portfolio Stylesheet -->\n";
	}
}
add_action('wp_head', 'WPPortfolio_renderCSS');



/**
 * Turn the portfolio of websites in the database into a single page containing details and screenshots using the [wp-portfolio] shortcode.
 * @param $atts The attributes of the shortcode.
 * @return String The updated content for the post or page.
 */
function WPPortfolio_convertShortcodeToPortfolio($atts)
{	
	// Process the attributes
	extract(shortcode_atts(array(
		'groups' 		=> '',
		'hidegroupinfo' => 0,
	), $atts));
	
	// If hidegroupinfo is 1, then hide group details by passing in a blank template to the render portfolio function
	$grouptemplate = false; // If false, then default group template is used.
	if ($atts['hidegroupinfo'] == 1) {
		$grouptemplate = "&nbsp;";
	}
	
	return WPPortfolio_getAllPortfolioAsHTML($atts['groups'], false, $grouptemplate);
}
add_shortcode('wp-portfolio', 'WPPortfolio_convertShortcodeToPortfolio');



/**
 * Method to get the portfolio using the specified list of groups and return it as HTML.
 * 
 * @param $groups The comma separated string of group IDs to show.
 * @param $template_website The template used to render each website. If false, the website template defined in the settings is used instead.
 * @param $template_group The template used to render each group header. If false, the group template defined in the settings is used instead.
 * @return String The HTML which contains the portfolio as HTML.
 */
function WPPortfolio_getAllPortfolioAsHTML($groups = '', $template_website = false, $template_group = false)
{
	// Get portfolio from database
	global $wpdb;
	$websites_table = $wpdb->prefix . TABLE_WEBSITES;
	$groups_table   = $wpdb->prefix . TABLE_WEBSITE_GROUPS;		
		
	// Determine if we only want to show certain groups
	$WHERE_CLAUSE = "";
	if ($groups)
	{ 
		$selectedGroups = explode(",", $groups);
		foreach ($selectedGroups as $possibleGroup)
		{
			// Some matches might be empty strings
			if ($possibleGroup > 0) {
				$WHERE_CLAUSE .= "$groups_table.groupid = '$possibleGroup' OR ";
			}
		}
	} // end of if ($groups)
		
	// Add initial where if needed
	if ($WHERE_CLAUSE)
	{
		// Remove last OR to maintain valid SQL
		if (substr($WHERE_CLAUSE, -4) == ' OR ') {
			$WHERE_CLAUSE = substr($WHERE_CLAUSE, 0, strlen($WHERE_CLAUSE)-4);
		}				
		
		$WHERE_CLAUSE = "WHERE ". $WHERE_CLAUSE;
	}
		
			
	// Get website details, merge with group details
	$SQL = "SELECT * FROM $websites_table
			LEFT JOIN $groups_table ON $websites_table.sitegroup = $groups_table.groupid
			$WHERE_CLAUSE
		 	ORDER BY grouporder, groupname, sitename";			
					
	$wpdb->show_errors();
	$websites = $wpdb->get_results($SQL, OBJECT);

	// If we've got websites to show, then render into HTML
	if ($websites) {
		$portfolioHTML = WPPortfolio_renderPortfolio($websites, $template_website, $template_group);
	} else {
		$portfolioHTML = false;
	}
	
	return $portfolioHTML;
}




/**
 * Method to get a random selection of websites from the portfolio using the specified list of groups and return it as HTML. No group details are 
 * returned when showing a random selection of the portfolio.
 * 
 * @param $groups The comma separated string of group IDs to use to find which websites to show. If false, websites are selected from the whole portfolio.
 * @param $count The number of websites to show in the output.
 * @param $template_website The template used to render each website. If false, the website template defined in the settings is used instead.
 * @return String The HTML which contains the portfolio as HTML.
 */
function WPPortfolio_getRandomPortfolioSelectionAsHTML($groups = '', $count = 3, $template_website = false)
{
	// Get portfolio from database
	global $wpdb;
	$websites_table = $wpdb->prefix . TABLE_WEBSITES;
	$groups_table   = $wpdb->prefix . TABLE_WEBSITE_GROUPS;		
		
	// Validate the count is a number
	$count = $count + 0;
	if ($count == 0) {
		$count = 3;
	}
	
	// Determine if we only want to get websites from certain groups
	$WHERE_CLAUSE = "";
	if ($groups)
	{ 
		$selectedGroups = explode(",", $groups);
		foreach ($selectedGroups as $possibleGroup)
		{
			// Some matches might be empty strings
			if ($possibleGroup > 0) {
				$WHERE_CLAUSE .= "$groups_table.groupid = '$possibleGroup' OR ";
			}
		}
	} // end of if ($groups)
		
	// Add initial where if needed
	if ($WHERE_CLAUSE)
	{
		// Remove last OR to maintain valid SQL
		if (substr($WHERE_CLAUSE, -4) == ' OR ') {
			$WHERE_CLAUSE = substr($WHERE_CLAUSE, 0, strlen($WHERE_CLAUSE)-4);
		}				
		
		$WHERE_CLAUSE = "WHERE ". $WHERE_CLAUSE;
	}
		
			
	// Get website details, merge with group details
	$SQL = "SELECT * FROM $websites_table
			LEFT JOIN $groups_table ON $websites_table.sitegroup = $groups_table.groupid
			$WHERE_CLAUSE
		 	ORDER BY RAND()
		 	LIMIT $count
		 	";			
					
	$wpdb->show_errors();
	$websites = $wpdb->get_results($SQL, OBJECT);

	// If we've got websites to show, then render into HTML. Use blank group to avoid rendering group details.
	if ($websites) {
		$portfolioHTML = WPPortfolio_renderPortfolio($websites, $template_website, ' ');
	} else {
		$portfolioHTML = false;
	}
	
	return $portfolioHTML;
}



/**
 * Convert the website details in the database object into the HTML for the portfolio.
 * @param $websites The website detail object.
 * @param $template_website The template used to render each website. If false, the website template defined in the settings is used instead.
 * @param $template_group The template used to render each group header. If false, the group template defined in the settings is used instead.
 * @return String The HTML for the portfolio page.
 */
function WPPortfolio_renderPortfolio($websites, $template_website = false, $template_group = false)
{
	if (!$websites)
		return false;
			
	// Just put some space after other content before rendering portfolio.	
	$content .= "\n\n";			

	// Used to track what group we're working with.
	$prev_group = "";
	
	// Get templates to use for rendering the website details. Use the defined options if the parameters are false.
	if (!$template_website) {
		$setting_template_website = stripslashes(get_option('WPPortfolio_setting_template_website'));
	} else {
		$setting_template_website = $template_website;		
	}
	
	if (!$template_group) {
		$setting_template_group = stripslashes(get_option('WPPortfolio_setting_template_group'));						
	} else {
		$setting_template_group = $template_group;		
	}
	 	
	
	// Render all the websites, but look after different groups
	foreach ($websites as $websitedetails)
	{
		// If we're rendering a new group, then show the group name and description 
		if ($prev_group != $websitedetails->groupname)
		{
			// Replace group name and description.					
			$renderedstr = WPPortfolio_replaceString(WPP_STR_GROUP_NAME, stripslashes($websitedetails->groupname), $setting_template_group);
			$renderedstr = WPPortfolio_replaceString(WPP_STR_GROUP_DESCRIPTION, stripslashes($websitedetails->groupdescription), $renderedstr);
			
			// Update content with templated group details
			$content .= "\n\n$renderedstr\n";
		}
		
		// Get the image thumbnail and generate the image tag. 
		
		
		// Render the website details
		$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_NAME, 		 	stripslashes($websitedetails->sitename), $setting_template_website);
		$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_URL, 		 	stripslashes($websitedetails->siteurl), $renderedstr);
		$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_DESCRIPTION, 	stripslashes($websitedetails->sitedescription), $renderedstr);
		
		// Handle the thumbnails - use custom if provided.
		if ($websitedetails->customthumb) {			
			$imagetag = '<img src="'.WPPortfolio_getAdjustedCustomThumbnail($websitedetails->customthumb).'"/>';
		} else {
			$imagetag = '<img src="'.WPPortfolio_getThumbnail($websitedetails->siteurl).'"/>';
		}		
		$renderedstr = WPPortfolio_replaceString(WPP_STR_WEBSITE_THUMBNAIL, $imagetag, $renderedstr);
		
		
		$content .= "\n$renderedstr\n";
		
		// If fetching thumbnails, this might take a while. So flush.
		flush();
		
		// Track the groups
		$prev_group = $websitedetails->groupname;
	}
	
	// Credit link on portfolio. 
	if (get_option('WPPortfolio_setting_show_credit') == "on") {				
		$content .= "<div style=\"font-size: 8pt; font-family: Verdana; float: right;\">Created using <a href=\"http://wordpress.org/extend/plugins/wp-portfolio\">WP Portfolio</a> by <a href=\"http://www.danharrison.co.uk/\">Dan Harrison</a></div>";
	}
				
	// Add some space after the portfolio HTML 
	$content .= "\n\n";
	
	return $content;
}


/**
 * Gets the thumbnail for the specified website, stores it in the cache, and then returns the 
 * relative path to the cached image.
 * 
 * @param $url The URL to get the thumbnail for.
 * @param $size_override If specified, use this size rather than the size in the settings.
 * @return String The full URL for the thumbnail stored in the cache.
 */
function WPPortfolio_getThumbnail($url, $size_override = false)
{
	// Determine cache directory
	$actualThumbPath = WPPortfolio_getThumbPathActualDir();
			
	// Create cache directory if it doesn't exist
	WPPortfolio_createCacheDirectory();
	
	// Get all the options from the database for the thumbnail    
    $args["STWAccessKeyId"] = stripslashes(get_option('WPPortfolio_setting_stw_access_key'));
    $args["stwu"] 			= stripslashes(get_option('WPPortfolio_setting_stw_secret_key'));
    $setting_fetch_method   = stripslashes(get_option('WPPortfolio_setting_fetch_method'));
        
    // If $size_override is specified, then use that size rather than the size stored in the settings.
    if ($size_override) {
    	$args["Size"] = $size_override;
    } else { 
    	$args["Size"] = stripslashes(get_option('WPPortfolio_setting_stw_thumb_size'));
    }
    
    // Try to grab the thumbnail
    $imagefile = WPPortfolio_getCachedThumbnail($url, 
    								$actualThumbPath, 
    								WPPortfolio_getThumbPathURL(),  
    								WPPortfolio_getPendingThumbURLPath(), 
    								$args, 
    								false, 
    								$setting_fetch_method);    
        
    return $imagefile;
}

/**
 * Fetch a custom thumbnail URL and resize it to match the maximum dimensions defined by the required thumbnail sizes.
 * @param $imageurl The image URL.
 * @param $size_override If specified, the standard size to use to resize the custom thumbnail.
 * @param $forceUpdate If true, force the update of the cached thumbnail.
 * @return String The full path of the image to render.
 */
function WPPortfolio_getAdjustedCustomThumbnail($imageurl, $size_override = false, $forceUpdate = false) 
{
	// Determine cache directory
	$actualThumbPath = WPPortfolio_getThumbPathActualDir();
			
	// Create cache directory if it doesn't exist
	WPPortfolio_createCacheDirectory();

	// Get all the options from the database for the thumbnail handling
    $setting_fetch_method   = stripslashes(get_option('WPPortfolio_setting_fetch_method'));

    // If $size_override is specified, then use that size rather than the size stored in the settings.
    if ($size_override) {
    	$maxsize = $size_override;
    } else { 
    	$maxsize = stripslashes(get_option('WPPortfolio_setting_stw_thumb_size'));
    }    
        
    // Use arguments to work out the target filename
	$filename = md5($imageurl.$maxsize).".jpg";
	$filepath = trailingslashit($actualThumbPath).$filename;
	
	// Work out if we need to update the cached and resized thumbnail
	if ($forceUpdate || WPPortfolio_cacheFileExpired($filepath))
	{		
		WPPortfolio_downloadRemoteImageToLocalPath($imageurl, $filepath, $setting_fetch_method);
		
		// Now we've got the image, resize it.
		if (file_exists($filepath))
		{
			// Turn the standard sizes into actual pixel sizes for the resizing function.	
			switch($maxsize) {
				case "sm":
					$maxWidth = 120;
					$maxHeight = 90;
					break;
					
				case "lg":
					$maxWidth = 200;
					$maxHeight = 150;
					break;
					
				// xlg
				default:
					$maxWidth = 320;
					$maxHeight = 240;
					break;
			}
			
			WPPortfolio_resizeImage($imageurl, $filepath, $maxWidth, $maxHeight);
		}
	}
    
	// File downloaded successfully
	if (file_exists($filepath)) {
		$webFilePath = WPPortfolio_getThumbPathURL();		
		return "$webFilePath/$filename";
	}
	// Something went wrong, so return default image 
	else {
		$pendingThumbPath = WPPortfolio_getPendingThumbURLPath();
		return "$pendingThumbPath$maxsize.jpg";
	}
}

/**
 * Determine if specified file has expired from the cache.
 * @param $filepath The full path of the file to check for in the cache.
 * @return Boolean True if the file has expired or no longer exists, or false if the file is still valid.
 */
function WPPortfolio_cacheFileExpired($filepath)
{
	// Use setting to check age of files.
	$setting_cache_days	= stripslashes(get_option('WPPortfolio_setting_cache_days')) + 0;
	
	// If cached thumbnails never expire, then just check if file exists or not.
	if ($setting_cache_days == 0) {
		return (!file_exists($filepath));
	} 
	// If thumbnails are allowed to expire, then check age of files and if file exists.
	else {	
		$cutoff = time() - 3600 * 24 * $setting_cache_days;
		return (!file_exists($filepath) || filemtime($filepath) <= $cutoff);	
	}
}

/**
 * Resize the specified image using the maximum dimensions provided as arguments.
 * @param $imagepath The actual file path of the image to resize.
 * @param $maxWidth The maximum width to resize the image to.
 * @param $maxHeight The maximum height to resize the image to.
 * @return unknown_type
 */
function WPPortfolio_resizeImage($originalImage, $imagepath, $maxWidth = 120, $maxHeight = 80)
{
	// What sort of image? Check the extension from the original image URL to determine the original 
	// image format, as we can't use the extension of the file in the cache, as that has already been 
	// changed to jpg.  
	$type = strtolower(substr(strrchr($originalImage, '.'), 1));
	
	$quality = 0;  
	switch ($type)  
	{     
	case 'png':  
		$image_create_func = 'ImageCreateFromPNG';  
		$image_save_func = 'ImagePNG';  
 	
		// Compression Level: from 0  (no compression) to 9  
		$quality = 0;  
		break;  
		
	case 'gif':  
		$image_create_func = 'ImageCreateFromGIF';  
		$image_save_func = 'ImageGIF';   
		break;   
	 
	// Also handles jpeg
	default:  
		$image_create_func = 'ImageCreateFromJPEG';  
		$image_save_func = 'ImageJPEG';    

		// Best Quality: 100  
		$quality = 100;  
	}  
	
	// Load image into GD (now that it's stored locally)
	$img = $image_create_func($imagepath); 
	
	// Resize image to max size 
	$newimg = WPPortfolio_resizeImageResource($img, $maxWidth, $maxHeight);
		
	// Save using quality parameter if available
	if (isset($quality)) {  
		$image_save_func($newimg, $imagepath, $quality);  
	}  
	else {  
		$image_save_func($newimg, $imagepath);  
	}  	
	
	// All done, clean up
	imagedestroy($img);
	imagedestroy($newimg);
		
}

/**
 * Resize the specified image resource to the maximum desired width and height.
 * @param $imgresource The image resource to resize to the maximum desired width and height.
 * @param $desired_max_width The desired maximum width.
 * @param $desired_max_height The desired maximum height.
 * @return Image Resource The resized image resource.
 */
function WPPortfolio_resizeImageResource($imgresource, $desired_max_width, $desired_max_height)
{
	$old_width  = imageSX($imgresource);
	$old_height = imageSY($imgresource);
	$new_width  = $old_width;
	$new_height = $old_height;
	
	// Check that image is already larger than needed, i.e. so needs resizing.
	if ($old_width > $desired_max_width  || $old_height > $desired_max_height)
	{
		// Work out the aspect ratio, as we need to ensure the image stays in 
		// the correct dimensions
		$aspect_ratio = $old_width / $old_height;
		
		// Use $desired_max_width as the new_width, and change new_height using 
		// the aspect ratio
		$new_width = $desired_max_width;
		$new_height = $new_width / $aspect_ratio;
		
		// Just check that the new height is actually below the max height, if not
		// adjust the sizes again to ensure it is
		if ($new_height > $desired_max_height)
		{
			
			$new_height = $desired_max_height;
			$new_width = $new_height * $aspect_ratio;
		}
	}
	
	$image_resized = ImageCreateTrueColor($new_width, $new_height);
	imagecopyresampled($image_resized, $imgresource, 0, 0, 0, 0, $new_width, $new_height, $old_width, $old_height);
	
	return $image_resized;
}


/**
 * Create the cache directory if it doesn't exist.
 */
function WPPortfolio_createCacheDirectory()
{
	// Cache directory
	$actualThumbPath = WPPortfolio_getThumbPathActualDir();
			
	// Create cache directory if it doesn't exist	
	if (!file_exists($actualThumbPath)) {
		@mkdir($actualThumbPath, 0777, true);		
	} else {
		// Try to make the directory writable
		@chmod($actualThumbPath, 0777);
	}
}

/**
 * Gets the full directory path for the thumbnail directory with a trailing slash.
 * @return String The full directory path for the thumbnail directory.
 */
function WPPortfolio_getThumbPathActualDir() {
	return trailingslashit(trailingslashit(WP_PLUGIN_DIR).WPP_THUMBNAIL_PATH);	
}

/**
 * Gets the full URL path for the thumbnail directory with a trailing slash.
 * @return String The full URL for the thumbnail directory.
 */
function WPPortfolio_getThumbPathURL() {
	return trailingslashit(trailingslashit(WP_PLUGIN_URL).WPP_THUMBNAIL_PATH);
}

/**
 * Get the full URL path of the pending thumbnails.
 * @return String The full URL path of the pending thumbnails.
 */
function WPPortfolio_getPendingThumbURLPath() {
	return trailingslashit(WP_PLUGIN_URL).WPP_THUMB_DEFAULTS;
}

/**
 * Function: WPPortfolio_showMessage();
 *
 * Shows settings saved message.
 */
function WPPortfolio_showMessage($message = "Settings saved.", $errormsg = false)
{
	if ($errormsg) {
		echo '<div id="message" class="error">';
	}
	else {
		echo '<div id="message" class="updated fade">';
	}

	echo "<p><strong>$message</strong></p></div>";
}

/**
 * Function: WPPortfolio_showRedirectionMessage();
 *
 * Shows settings saved and page being redirected message.
 */
function WPPortfolio_showRedirectionMessage($message, $target, $delay)
{
?>
	<div id="message" class="updated fade">
		<p>
			<strong><?php echo $message; ?><br /><br />
			Redirecting in <?php echo $delay ?> seconds. Please click <a href="<?php echo $target; ?>">here</a> if you do not wish to wait.</strong>
		</p>
	</div>
	
	<SCRIPT language="JavaScript">
    <!--
            function getgoing() {
                     top.location="<?php echo $target; ?>";
            }

            if (top.frames.length==0) {
                setTimeout('getgoing()',<?php echo $delay * 1000 ?>);
            }
	//-->
	</SCRIPT>
	<?php
}

/**
 * Determine if the specified key is valid, i.e. containing only letters and numbers.
 * @param $key The key to check
 * @return Boolean True if the key is valid, false otherwise.
 */
function WPPortfolio_isValidKey($key) 
{
	// Ensure the key only contains letters and numbers
	return preg_match('/^[a-z0-9A-Z]+$/', $key);
}

/**
 * Recursively delete a directory
 *
 * @param string $dir Directory name
 * @param boolean $deleteRootToo Delete specified top-level directory as well
 */
function WPPortfolio_unlinkRecursive($dir, $deleteRootToo)
{
    if(!$dh = @opendir($dir)) {
        return;
    }
    while (false !== ($obj = readdir($dh)))
    {
        if($obj == '.' || $obj == '..') {
            continue;
        }

        if (!@unlink($dir . '/' . $obj)) {
            WPPortfolio_unlinkRecursive($dir.'/'.$obj, true);
        }
    }

    closedir($dh);
   
    if ($deleteRootToo) {
        @rmdir($dir);
    }
   
    return;
} 

/**
 * Replace all occurances of the search string with the replacement. Uses alternative for str_ireplace if not available.
 * @param $searchstr The string to search for.
 * @param $replacestr The string to replace the search string with.
 * @param $haystack The string to search.
 * @return String The text with the replaced string.
 */
function WPPortfolio_replaceString($searchstr, $replacestr, $haystack) {
	
	// Faster, but in PHP5.
	if (function_exists("str_ireplace")) {
		return str_ireplace($searchstr, $replacestr, $haystack);
	}
	// Slower but handles PHP4
	else { 
		return preg_replace("/$searchstr/i", $replacestr, $haystack);
	}
}



/**
 * Get a thumbnail, caching it first if possible.
 * 
 * @param $url The URL to get a thumbnail for.
 * @param $actualThumbPath The actual path on the server to store the cached thumbnail.
 * @param $webFilePath The web URL path for the thumbnails that relates to the actual file path.
 * @param $pendingThumbPath The path for images when a thumbnail cannot be loaded.
 * @param $args The arguments that contain the access ID and size info to get the thumbnail.
 * @param $forceUpdate If true, force the retrieval over the web of the thumbnail.
 * @param $fetchMethod The method used to fetch a thumbnail (either curl or fopen).
 * 
 * @return String The full path of the image to render.
 */
function WPPortfolio_getCachedThumbnail($url, $actualThumbPath, $webFilePath, $pendingThumbPath,
							$args = null, $forceUpdate = false, $fetchMethod = "curl")
{
	// Don't try something if not enough arguements.
	if (!$args || !$url || !$actualThumbPath || !$webFilePath)
		return false;

	// Use arguements to work out the target filename
	$filename = md5($url.serialize($args)).".jpg";
	$filepath = trailingslashit($actualThumbPath).$filename;
	$size = $args['Size'];
	
	// Work out if we need to update the cached and resized thumbnail
	if ($forceUpdate || WPPortfolio_cacheFileExpired($filepath))
	{
		// Try to download the file
		if ($jpgurl = WPPortfolio_checkWebsiteThumbnailCaptured($url, $args, $fetchMethod)) {		
			WPPortfolio_downloadRemoteImageToLocalPath($jpgurl, $filepath, $fetchMethod);
		}
	}

	// File downloaded successfully
	if (file_exists($filepath)) {
		return "$webFilePath/$filename";
	}
	// Something went wrong, so return default image 
	else {
		return "$pendingThumbPath$size.jpg";
	}
}

/**
 * Method that checks that the thumbnail for the specified website exists. 
 * @param $url The website to get the thumbnail for.
 * @param $args The arguements used for the HTTP request.
 * @param $fetchMethod The method used to fetch a thumbnail (either curl or fopen). 
 * 
 * @return The URL of the image file from the server if the request was successful, false otherwise.
 */
function WPPortfolio_checkWebsiteThumbnailCaptured($url, $args = null, $fetchMethod = "curl")
{
	$args = is_array($args) ? $args : array();	
	
	// Don't try something if not enough arguements.
	if (!$args || !$url)
		return false;
		
    $args["Service"] = "ShrinkWebUrlThumbnail";
    $args["Action"] = "Thumbnail";		
	$args["stwUrl"] = $url; // now url is added to the arguements
	
	$request_url = urldecode("http://www.shrinktheweb.com/xino.php?".http_build_query($args,'','&'));	// avoid &amp;
	
	// Check that the thumbnail exists, from the STW server
	// Use cURL if possible
	if ($fetchMethod == "curl" && function_exists('curl_init')) 
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $request_url);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1); // return into a variable
		$remotedata = curl_exec($ch);
		curl_close($ch);		
	}
	// This may be disabled as a security measure
	else
	{
		$remotedata = file_get_contents($request_url);
	}
	   
	$regex = '/<[^:]*:Thumbnail\\s*(?:Exists=\"((?:true)|(?:false))\")?[^>]*>([^<]*)<\//';
	if (preg_match($regex, $remotedata, $matches) == 1 && $matches[1] == "true")
		return $matches[2];
		
	// TODO Handle the case when there's an error.  That's when the preg_match fails.
	// else echo $line;

	return false;
}

/**
 * Method to get image at the specified remote URL and attempt to save it to the specifed local path.
 * @param $remoteURL The URL of the remote image to download.
 * @param $localPath The path to use to store the image locally.
 * @param $fetchMethod The method used to fetch a thumbnail (either curl or fopen). 
 */
function WPPortfolio_downloadRemoteImageToLocalPath($remoteURL, $localPath, $fetchMethod = "curl")
{
	if ($fetchMethod == "curl" && function_exists('curl_init'))
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $remoteURL);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10); // Timeout in 10 seconds		
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER,1); // return into a variable
		$imagedata = curl_exec($ch);		
	}
	else {		
		$imagedata = file_get_contents($remoteURL);
	}

	// Only save data if we managed to get the file contents
	if ($imagedata) {
		$localFileHandle = fopen($localPath, "w+");
		fputs($localFileHandle, $imagedata);
		fclose($localFileHandle);
	} else {	
		// Try to delete file if download failed.
		if (file_exists($localPath)) {	
			@unlink($localPath);
		}
	}
}



/**
 * Get the details for the specified group ID.
 * @param $groupid The ID of the group to get the details for.
 * @return Array An array of the group details.
 */
function WPPortfolio_getGroupDetails($groupid) 
{
	global $wpdb;
	$table_name = $wpdb->prefix . TABLE_WEBSITE_GROUPS;
	
	$SQL = "SELECT * FROM $table_name 
			WHERE groupid = '".$wpdb->escape($groupid)."' LIMIT 1";
	
	// We need to strip slashes for each entry.
	return WPPortfolio_cleanSlashesFromArrayData($wpdb->get_row($SQL, ARRAY_A));
}

/**
 * Get the details for the specified Website ID.
 * @param $siteid The ID of the Website to get the details for.
 * @return Array An array of the Website details.
 */
function WPPortfolio_getWebsiteDetails($siteid) 
{
	global $wpdb;
	$table_name = $wpdb->prefix . TABLE_WEBSITES;
	
	$SQL = "SELECT * FROM $table_name 
			WHERE siteid = '".$wpdb->escape($siteid)."' LIMIT 1";

	// We need to strip slashes for each entry.
	return WPPortfolio_cleanSlashesFromArrayData($wpdb->get_row($SQL, ARRAY_A));
}

/**
 * Remove all slashes from all fields from data retrieved from the database.
 * @param $data The data array from the database.
 * @return Array The cleaned array.
 */
function WPPortfolio_cleanSlashesFromArrayData($data)
{
	if (count($data) > 0) {
		foreach ($data as $datakey => $datavalue) {
			$data[$datakey] = stripslashes($datavalue);
		}
	}
	
	return $data;
}
?>